/*global console, webapis*/
/*jslint evil: true */
/*
* Copyright (c) 2015 Samsung Electronics Co., Ltd.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are
* met:
*
* * Redistributions of source code must retain the above copyright
* notice, this list of conditions and the following disclaimer.
* * Redistributions in binary form must reproduce the above
* copyright notice, this list of conditions and the following disclaimer
* in the documentation and/or other materials provided with the
* distribution.
* * Neither the name of Samsung Electronics Co., Ltd. nor the names of its
* contributors may be used to endorse or promote products derived from
* this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
var SAPConnection = function (appName, role, onRequest, onDebug) {
    'use strict';
    this.appName = appName;

    this.role = role || 'PROVIDER';
    this.onRequest = onRequest || new Function();
    this.onDebug = onDebug || console.log;

    this.SAAgent = {};
};

SAPConnection.prototype.requestOnSuccess = function requestOnSuccess(agents) {
    'use strict';
    var self = this,
        i = 0;

    for (i; i < agents.length; i += 1) {
        if (agents[i].role === this.role) {
            this.onDebug('Ready to connect');
            this.SAAgent = agents[i];
            break;
        }
    }

    this.SAAgent.setServiceConnectionListener({
        onrequest: function onrequest(peerAgent) {
            self.onDebug('Peer requesting conncetion');

            if (peerAgent.appName === self.appName) {
                self.SAAgent.acceptServiceConnectionRequest(peerAgent);
                self.onDebug('Connection accepted');
            } else {
                self.SAAgent.rejectServiceConnectionRequest(peerAgent);
                self.onDebug('Connection rejected');
            }
        },
        onconnect: function onconnect(socket) {
            self.onDebug('Connection established');

            function onConnectionLost(reason) {
                self.onDebug('Disconnected');
            }

            function dataOnReceive(channelId, data) {
                self.onRequest(channelId, socket, JSON.parse(data));
            }

            socket.setSocketStatusListener(onConnectionLost);
            socket.setDataReceiveListener(dataOnReceive);
        },
        onerror: function onerror(errorCode) {
            self.onDebug('Connection error<br/>' + errorCode);
        }
    });
};

SAPConnection.prototype.requestOnError = function requestOnError(e) {
    'use strict';
    this.onDebug('Agent request error<br/>' + e.message);
};

SAPConnection.prototype.connect = function connect() {
    'use strict';
    webapis.sa.requestSAAgent(
        this.requestOnSuccess.bind(this),
        this.requestOnError.bind(this)
    );
};
