/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdlib.h>
#include <string.h>
#include <dlog.h>

char *getStringValue(char *json_string, char *key, int *val)
{

	int len = 0;
	char *start;
	char *result;

	start = strstr(json_string, key);
	if (!start) {
		//Key doesn't exist
		return NULL;
	}

	start = start + strlen(key) + strlen("\":\"");

	while (start[len] != '"' && start[len]) {
		++len;
	}


	result = (char *)calloc(1, sizeof(char) * (len + 1));
	if (NULL == result) {
		return NULL;
	}

	strncpy(result, start, (size_t)len);
	*val = (int)(start + len);
	return result;
}

int getIntegerValue(char *json_string, char *key)
{
	int len = 0;
	char *start;

	int result = 0;
	int isNegative = 0;

	start = strstr(json_string, key);
	if (!start) {
		//Key doesn't exist
		return 0x0LL;
	}
	start = start + strlen(key) + strlen("\":");
	if (*start == '-') {
		//The numbe is -ve
		++start;
		isNegative = 1;
	}
	while (start[len] != ',' && start[len] != '}' && start[len]) {
		result = 10 * result + (int)(start[len] - '0');
		++len;
	}
	if (isNegative) {
		result = -result;
	}
	return result;
}

char *parse_msg_id(char *req_msg)
{
	int len;
	char *msg_id = getStringValue(req_msg, "msgId", &len);

	return msg_id;
}

int parse_image_id(char *req_msg)
{
	return getIntegerValue(req_msg, "id");
}
