/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following disclaimer
 *       in the documentation and/or other materials provided with the
 *       distribution.
 *     * Neither the name of Samsung Electronics Co., Ltd. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

var SAAgent = {
    '/sample/multiplicity1': null,
    '/sample/multiplicity2': null
};
var SASocket = {
    '/sample/multiplicity1': null,
    '/sample/multiplicity2': null
};
var CHANNELID = {
    '/sample/multiplicity1': 1000,
    '/sample/multiplicity2': 2000
};
var ProviderAppName = "MultiplicityProviders";

function createHTML(log_string)
{
    var content = document.getElementById("toast-content");
    content.textContent = log_string;
    tau.openPopup("#toast");
}

function onerror(err) {
    console.log("err [" + err + "]");
}

var agentCallback = function(id) {
    return {
        onconnect : function(socket) {
            SASocket[id] = socket;
            createHTML("Multiplicity " + id + " Connection established with RemotePeer");
            SASocket[id].setSocketStatusListener(function(reason){
                console.log("Service " + id + " connection lost, Reason : [" + reason + "]");
                disconnect(id);
            });
            SASocket[id].setDataReceiveListener(onreceive.bind(this, id));
        },
        onerror : onerror
    };
};

var peerAgentFindCallback = function(id) {
    return {
        onpeeragentfound : function(peerAgent) {
            try {
                if (peerAgent.appName == ProviderAppName) {
                    SAAgent[id].setServiceConnectionListener(agentCallback(id));
                    SAAgent[id].requestServiceConnection(peerAgent);
                } else {
                    createHTML("Not expected app!! : " + peerAgent.appName);
                }
            } catch(err) {
                console.log("exception [" + err.name + "] msg[" + err.message + "]");
            }
        },
        onerror : onerror
    };
}

function onsuccess(id, agents) {
    var agentsLen = agents.length,
        found = false,
        i = 0;

    for(i; i < agentsLen; i += 1) {
        if (agents[i].id === id) {
            found = true;
            try {
                SAAgent[id] = agents[i];

                SAAgent[id].setPeerAgentFindListener(peerAgentFindCallback(id));
                SAAgent[id].findPeerAgents();
            } catch(err) {
                console.log("exception [" + err.name + "] msg[" + err.message + "]");
            }
        }
    }

    if (!found) {
        createHTML("Not found SAAgent " + id + "!!");
    }
}

function connect(id) {
    if (SASocket[id]) {
        createHTML('Already connected!');
        return false;
    }
    try {
        webapis.sa.requestSAAgent(onsuccess.bind(this, id), function (err) {
            console.log("err [" + err.name + "] msg[" + err.message + "]");
        });
    } catch(err) {
        console.log("exception [" + err.name + "] msg[" + err.message + "]");
    }
}

function disconnect(id) {
    try {
        if (SASocket[id] != null) {
            SASocket[id].close();
            SASocket[id] = null;
            createHTML("closeConnection " + id);
        }
    } catch(err) {
        console.log("exception [" + err.name + "] msg[" + err.message + "]");
    }
}

function onreceive(id, channelId, data) {
    createHTML(data);
}

function fetch(id) {
    try {
        SASocket[id].sendData(CHANNELID[id], "Multiplicity " + id);
    } catch(err) {
        console.log("exception [" + err.name + "] msg[" + err.message + "]");
    }
}

window.onload = function () {
    // add eventListener for tizenhwkey
    document.addEventListener('tizenhwkey', function(e) {
        if(e.keyName == "back")
            tizen.application.getCurrentApplication().exit();
    });
};

(function(tau) {
    var toastPopup = document.getElementById('toast');
    toastPopup.addEventListener('popupshow', function(ev){
        setTimeout(function(){tau.closePopup();}, 3000);
    }, false);
})(window.tau);